/****************************************************************************\
* mkimg - Create KNOPPIX image file from within another OS                   *
* (C) Klaus Knopper 2005 <knoppix@knopper.net>                               *
*                                                                            *
* This program is free software; you can redistribute it and/or modify       *
* it under the terms of the GNU General Public License version 2             *
* as published by the Free Software Foundation.                              *
\****************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <memory.h>
#include <ctype.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>

#define MB 0x100000

#define COPYRIGHT "(C) KNOPPER.NET 2005\n\r\n\r"

#ifdef GERMAN
#define TITLE \
 "KNOPPIX IMAGE-DATEI EINRICHTEN\n\r" \
 "==============================\n\r\n\r"
#define HELP \
 "Dieses Programm richtet eine Image-Datei bzw. ein virtuelles Laufwerk\n\r" \
 "fuer KNOPPIX 3.8 oder hoeher auf Ihrer Festplatte ein.\n\r\n\r" \
 "Mit Hilfe dieser Image-Datei koennen Sie persoenliche Einstellungen\n\r" \
 "und Daten unter KNOPPIX abspeichern.\n\r\n\r"
#define MSG1 \
 "Bitte Laufwerkbuchstaben eingeben (C fuer erste Partition): "
#define MSG2 \
 "BITTE STELLEN SIE SICHER, DASS AUCH NACH DEM EINRICHTEN DES KNOPPIX-\n\r" \
 "IMAGE NOCH GENUEGEND FREIER PLATZ AUF LAUFWERK %c: VORHANDEN IST,\n\r" \
 "DAMIT IHR INSTALLIERTES BETRIEBSSYSTEM IN SEINER FUNKTION NICHT\n\r" \
 "BEEINTRAECHTIGT WIRD!\n\r\n\r" \
 "Bitte gewuenschte Groesse des KNOPPIX-Image in MB angeben\n\r" \
 "(mindestens 100!): "
#define ERROR_NO_OPEN "Kann Image-Datei nicht anlegen"
#define ERROR_NO_WRITE "Kann Image-Datei nicht schreiben"
#define ERROR_NO_MEM "Kann nicht genug Hauptspeicher belegen"
#define ERROR_FILE_EXISTS \
 "\n\rDie Datei %s existiert bereits (Groesse=%dMB).\n\r" \
 "Loeschen und neu anlegen? (Alle Daten im Image werden geloescht.) [j/N] "
#define WRITING "\n\rLege %s an... "
#define SUCCESS \
 "Fertig!\n\r\n\r" \
 "Sie koennen nun Ihr persistentes KNOPPIX Heimverzeichnis von KNOPPIX\n\r" \
 "aus einrichten.\n\r"
#define RETURN_TO_CONTINUE "\n\rEingabetaste zum Beenden."
#else
#define TITLE \
 "CREATE KNOPPIX IMAGE FILE\n\r" \
 "=========================\n\r\n\r"
#define HELP \
 "This program creates an image file (virtual disk drive) for KNOPPIX\n\r" \
 "on your harddisk.\n\r\n\r" \
 "Using this image file, you can save your personal settings and data\n\r" \
 "when running KNOPPIX.\n\r\n\r"
#define MSG1 \
 "Please enter drive letter (C for first disk partition): "
#define MSG2 \
 "PLEASE MAKE SURE THAT, AFTER INSTALLING THE KNOPPIX IMAGE, THERE IS\n\r" \
 "STILL ENOUGH FREE SPACE ON YOUR %c: DRIVE, SO THAT YOUR INSTALLED\n\r" \
 "OPERATING SYSTEM IS NOT AFFECTED BY LACK OF DISK SPACE!\n\r\n\r" \
 "Please enter the desired size for the KNOPPIX image in Megabytes\n\r" \
 "(must be at least 100!): "
#define ERROR_NO_OPEN "Cannot create image file"
#define ERROR_NO_WRITE "Cannot write to image file"
#define ERROR_NO_MEM "Cannot allocate enough memory"
#define ERROR_FILE_EXISTS \
 "\n\rThe file %s already exists (Size=%dMB).\n\r" \
 "Delete and recreate? (All data within the image will be erased.) [y/N] "
#define WRITING "\n\rCreating %s... "
#define SUCCESS \
 "Ready!\n\r\n\r" \
 "You can now setup your persistent KNOPPIX home directory from KNOPPIX.\n\r"
#define RETURN_TO_CONTINUE "\n\rHit enter to leave this program."
#endif

char *getline(void)
{
 static char buffer[32];
 return fgets(buffer, 30, stdin);
}

void cls(void)
{ printf("\033[H\033[J" TITLE); fflush(stdout); }

void end(int rc)
{ printf(RETURN_TO_CONTINUE); fflush(stdout); getline(); exit(rc); }

int menu1(int argc, char **argv)
{
 const char *msg=MSG1;
 cls(); printf(COPYRIGHT HELP);
 for(;;)
  {
   char *line;
   char c;
   int drive;
   printf(msg); fflush(stdout);
   if(!(line=getline())) end(-1);
   if(!sscanf(line, " %1c", &c)) end(0);
   drive = toupper(c);
   if(drive >= 'A' && drive <= 'Z') return drive;
   cls();
  }
 return 0;
}

unsigned int menu2(int drive, int argc, char **argv)
{
 const char *msg=MSG2;
 for(;;)
  {
   char *line;
   int size = 0;
   cls();
   printf(msg, drive); fflush(stdout);
   if(!(line=getline())) end(-1);
   if(!sscanf(line, " %d", &size)) end(0);
   if(size >= 100) return size;
  }
 return 0;
}

void checkifexists(char *path)
{
 struct stat s;
 char *line;
 char c;
 const char *msg=ERROR_FILE_EXISTS;
 if(stat(path, &s)) return;
 cls();
 printf(msg, path, s.st_size/MB); fflush(stdout);
 if(!(line=getline())) end(-1);
 if(!sscanf(line, " %1c", &c)) end(0);
 switch(toupper(c))
 {
  case 'J':
  case 'Y': chmod(path,438); unlink(path); return;
 }
 end(0);
}

int main(int argc, char **argv)
{
 int drive;
 unsigned int size;
 char path[] = "C:\\KNOPPIX.IMG";
 char *mem;
 int f, i;

 drive = menu1(argc, argv);
 path[0] = (char) drive;
 checkifexists(path);
 size  = menu2(drive, argc, argv);

 cls();


 if(!(mem=(char *)malloc(MB)))
  {
   perror(ERROR_NO_MEM);
   end(errno);
  }

 memset(mem,0,MB);

 unlink(path);
 if((f=open(path, O_WRONLY|O_CREAT))<0)
  {
   perror(ERROR_NO_OPEN);
   free(mem);
   end(-1);
  }

 chmod(path,438);

 printf(WRITING, path); fflush(stdout);

 for(i=0; i<size; i++)
 {
  if(write(f, mem, MB) != MB)
   {
    perror(ERROR_NO_WRITE);
    free(mem); close(f); unlink(path);
    end(-1);
   }
 }

 close(f);
 free(mem);
 printf(SUCCESS);

 end(0);

 return 0;
}
